@@ -39,15 +39,18 @@ from mpl_toolkits.mplot3d import Axes3D as axes_3d_t
 array_t = nmpy.ndarray
 
 axes_t = pypl.Axes
+axes_any_h = axes_t | axes_3d_t
 figure_t = pypl.Figure
 
 NewFigure = pypl.figure
 EnterMatplotlibEventLoop = pypl.show
 
+y_transform_h = Callable[[int | array_t], array_t]
+
 
 def FigureAndAxesFromBoth(
     figure: figure_t | None, axes: axes_t | axes_3d_t | None, dimension: int, /
-) -> tuple[figure_t, axes_t | axes_3d_t]:
+) -> tuple[figure_t, axes_any_h]:
     """"""
     if axes is None:
         if figure is None:
@@ -64,37 +67,39 @@ def FigureAndAxesFromBoth(
 
 
 def YTransformationsFromAxes(
-    axes: axes_t | axes_3d_t, domain_height: int, /
-) -> tuple[Callable[[array_t], array_t], Callable[[array_t], array_t]]:
+    axes: axes_any_h, domain_height: int, /
+) -> tuple[y_transform_h, y_transform_h]:
     """
     Note: For sites, an inverted y-axis (increasing to the bottom) is the normal, row/col convention for SKL-Graph, so
     TransformedY is the identity. However, it would flip vectors, so TransformedYForVector should indeed flip them.
     """
+    # nmpy.asarray: To be able to transform several y's at once
     if axes.yaxis_inverted():
         TransformedY = lambda y: y
         TransformedYForVector = lambda y: -nmpy.asarray(y)
     else:
         max_0 = domain_height - 1
-        # nmpy.asarray: To be able to transform several y's at once
         TransformedY = lambda y: max_0 - nmpy.asarray(y)
         TransformedYForVector = lambda y: y
 
     return TransformedY, TransformedYForVector
 
 
-def YTransformations(domain_height: int, /) -> Callable:
+def YTransformations(
+    domain_height: int, /
+) -> Callable[[axes_any_h], tuple[y_transform_h, y_transform_h]]:
     """"""
     return lambda _axs: YTransformationsFromAxes(_axs, domain_height)
 
 
 def ManagePlotOptions(
     figure: figure_t,
-    axes: axes_t | axes_3d_t,
+    axes: axes_any_h,
     should_block: bool,
     should_return_figure: bool,
     should_return_axes: bool,
     /,
-) -> figure_t | axes_t | axes_3d_t | tuple[figure_t, axes_t | axes_3d_t] | None:
+) -> figure_t | axes_t | axes_3d_t | tuple[figure_t, axes_any_h] | None:
     """"""
     # Preferably test not-axes_3d_t instead of yep-axes_t in case axes_3d_t is a subclass of axes_t
     if not isinstance(axes, axes_3d_t):
