@@ -103,27 +103,31 @@ def Plot(
         )
 
         if direction_style.show:
-            initial_direction = details.InitialDirection()
-            if initial_direction is not None:
-                dir_sites = tuple(
-                    nmpy.hstack((sites[idx_][0], sites[idx_][-1]))
-                    for idx_ in range(space_dim)
-                )
-                directions = list(zip(initial_direction, details.FinalDirection()))
-                directions[0], directions[1] = (
-                    directions[1],
-                    TransformedYForVector(directions[0]),
-                )
-                axes.quiver(
-                    *dir_sites,
-                    *directions,
-                    color=direction_style.color,
-                    linewidth=direction_style.size,
-                )
+            dir_sites = tuple(
+                nmpy.hstack((sites[idx_][0], sites[idx_][-1]))
+                for idx_ in range(space_dim)
+            )
+            directions = list(zip(details.InitialDirection(), details.FinalDirection()))
+            directions[0], directions[1] = (
+                directions[1],
+                TransformedYForVector(directions[0]),
+            )
+            axes.quiver(
+                *dir_sites,
+                *directions,
+                color=direction_style.color,
+                linewidth=direction_style.size,
+            )
 
         if label_style.show:
             middle_idx = sites[0].size // 2
-            position = tuple(_crd[middle_idx] for _crd in sites)
+            if sites[0].size % 2 == 0:
+                position = tuple(
+                    nmpy.mean(_crd[(middle_idx - 1) : (middle_idx + 1)]).item()
+                    for _crd in sites
+                )
+            else:
+                position = tuple(_crd[middle_idx] for _crd in sites)
             label = PlotEdgeLabel(
                 *position,
                 details.uid,
