@@ -40,16 +40,20 @@ from skl_graph.task.plot.base import (
     ManagePlotOptions,
     YTransformations,
     YTransformationsFromAxes,
+    axes_any_h,
     axes_3d_t,
     axes_t,
     figure_t,
+    y_transform_h,
 )
 from skl_graph.task.plot.edge import Plot as PlotEdges
 from skl_graph.task.plot.node import (
     Plot2DBranchNodes,
+    Plot2DEndNodes,
+    Plot2DNodeLabels,
     Plot3DBranchNodes,
+    Plot3DEndNodes,
     Plot3DNodeLabels,
-    PlotEndNodes,
     PositionsForPlotFromDetails,
     positions_as_dict_h,
 )
@@ -64,12 +68,12 @@ def Plot(
     /,
     *,
     figure: figure_t = None,
-    axes: axes_t | axes_3d_t = None,
+    axes: axes_any_h = None,
     mode: plot_mode_e = plot_mode_e.SKL_Pixel,
     should_block: bool = True,
     should_return_figure: bool = False,
     should_return_axes: bool = False,
-) -> figure_t | axes_t | axes_3d_t | tuple[figure_t, axes_t | axes_3d_t] | None:
+) -> figure_t | axes_t | axes_3d_t | tuple[figure_t, axes_any_h] | None:
     """"""
     if skl_graph.n_nodes < 1:
         return None
@@ -126,7 +130,7 @@ def PlotNetworkXGraph(
     graph: ntkx.MultiGraph,
     /,
     *,
-    NodePositions: Callable = None,
+    NodePositions: Callable[[axes_t], positions_as_dict_h] = None,
     node_colors: tuple[str, ...] = None,
     with_node_labels: bool = False,
     node_font_size: int = None,
@@ -134,11 +138,11 @@ def PlotNetworkXGraph(
     edge_labels: dict[str, str] = None,
     edge_font_size: int = None,
     figure: figure_t = None,
-    axes: axes_t | axes_3d_t = None,
+    axes: axes_t = None,
     should_block: bool = True,
     should_return_figure: bool = False,
     should_return_axes: bool = False,
-) -> figure_t | axes_t | axes_3d_t | tuple[figure_t, axes_t | axes_3d_t] | None:
+) -> figure_t | axes_t | axes_3d_t | tuple[figure_t, axes_t] | None:
     """
     Only 2-D graphs
     """
@@ -172,7 +176,9 @@ def PlotNetworkXGraph(
     )
 
 
-def NodePositionsForPlot(skl_graph: skl_graph_t, /) -> Callable:
+def NodePositionsForPlot(
+    skl_graph: skl_graph_t, /
+) -> Callable[[axes_any_h], positions_as_dict_h]:
     """"""
     YTransformationsForAxes = YTransformations(skl_graph.domain_shape[0])
 
@@ -206,9 +212,9 @@ def EdgeLabelsForPlot(skl_graph: skl_graph_t, /) -> dict[str, str]:
 def _PlotExplicitly(
     skl_graph: skl_graph_t,
     node_label_positions: positions_as_dict_h,
-    TransformedY: Callable[[array_t], array_t],
-    TransformedYForVector: Callable[[array_t], array_t],
-    axes: axes_t | axes_3d_t,
+    TransformedY: y_transform_h,
+    TransformedYForVector: y_transform_h,
+    axes: axes_any_h,
     mode: plot_mode_e,
     /,
 ) -> None:
@@ -224,18 +230,19 @@ def _PlotExplicitly(
         mode,
     )
 
-    # TODO: there is no distinction between end nodes and isolated nodes
+    if skl_graph.dim == 2:
+        PlotEndNodes = Plot2DEndNodes
+        PlotBranchNodes = Plot2DBranchNodes
+    else:
+        PlotEndNodes = Plot3DEndNodes
+        PlotBranchNodes = Plot3DBranchNodes
+    # Note: there is no distinction between end nodes and isolated nodes
     PlotEndNodes(
         skl_graph.nodes.data("details"),
         TransformedY,
         axes,
         skl_graph.node_styles.get(1, skl_graph.node_styles[None]),
     )
-
-    if skl_graph.dim == 2:
-        PlotBranchNodes = Plot2DBranchNodes
-    else:
-        PlotBranchNodes = Plot3DBranchNodes
     PlotBranchNodes(
         skl_graph.nodes.data("details"),
         skl_graph.degree,
@@ -246,26 +253,7 @@ def _PlotExplicitly(
 
     if skl_graph.label_styles[0].show:
         if skl_graph.dim == 2:
-            # NetworkX changes axes styles. Save here to set back afterwards.
-            axes_styles = {}
-            for axis, name in ((axes.xaxis, "x"), (axes.yaxis, "y")):
-                axes_styles[name] = {}
-                for level in ("minor", "major"):
-                    axes_styles[name][level] = axis.get_tick_params(which=level)
-            ntkx.draw_networkx_labels(
-                skl_graph,
-                ax=axes,
-                pos=node_label_positions,
-                font_size=int(round(skl_graph.label_styles[0].size)),
-                font_color=skl_graph.label_styles[0].color,
-            )
-            for name, styles in axes_styles.items():
-                for level, style in styles.items():
-                    axes.tick_params(
-                        axis=name,
-                        which=level,
-                        **style,
-                    )
+            Plot2DNodeLabels(skl_graph, node_label_positions, axes)
         else:
             Plot3DNodeLabels(
                 skl_graph, node_label_positions, axes, skl_graph.label_styles[0]
