@@ -83,14 +83,16 @@ class raw_edge_t:
 
     @property
     def n_sites(self) -> int:
-        """"""
+        """
+        Cannot be less than 2 since even an edge between an end node and a touching branch node will contain the end
+        node pixel and one branch node pixel.
+        """
         return self.sites[0].size
 
     def __str__(self) -> str:
         """"""
         return (
-            f"{self.__class__.__name__}:\n"
-            f"    Sites[{self.dim}-D]: {self.sites[0].size}"
+            f"{self.__class__.__name__}:\n" f"    Sites[{self.dim}-D]: {self.n_sites}"
         )
 
 
@@ -120,29 +122,94 @@ class edge_t(raw_edge_t):
 
         return transient.AsEdge()
 
+    @property
+    def arc_lengths(self) -> array_t:
+        """"""
+        output = nmpy.cumsum([0.0] + list(self.measures.segment_lengths))
+        output[-1] = self.measures.length  # To correct round-off errors
+        assert output[-1] > output[-2]  # To check above correction was not invalid
+
+        return output
+
     def AsCurve(self) -> spnt.PchipInterpolator | None:
         """"""
         cache_entry = self.AsCurve.__name__
 
         if cache_entry not in self._cache:
-            if self.n_sites > 1:
-                arc_lengths = nmpy.cumsum([0.0] + list(self.measures.segment_lengths))
+            if self.n_sites > 3:
                 self._cache[cache_entry] = _PchipInterpolator(
-                    arc_lengths, nmpy.array(self.sites)
+                    self.arc_lengths, nmpy.array(self.sites)
                 )
             else:
                 self._cache[cache_entry] = None
 
         return self._cache[cache_entry]
 
-    def InitialDirection(self) -> array_t | None:
+    def InitialDirection(self) -> array_t:
         """"""
         return self._Direction(self.InitialDirection)
 
-    def FinalDirection(self) -> array_t | None:
+    def FinalDirection(self) -> array_t:
         """"""
         return self._Direction(self.FinalDirection)
 
+    def Directions(self) -> tuple[array_t, array_t, array_t]:
+        """"""
+        cache_entry = self.Directions.__name__
+
+        if cache_entry not in self._cache:
+            as_curve = self.AsCurve()
+            if as_curve is None:
+                if self.n_sites > 1:
+                    sites_as_array = nmpy.array(self.sites)
+                    derivatives = nmpy.diff(sites_as_array, axis=1)
+                    norms = nmpy.linalg.norm(derivatives, axis=0, keepdims=True)
+                    derivatives /= norms
+                    last = derivatives[:, -1][:, None]
+                    self._cache[cache_entry] = (
+                        sites_as_array,
+                        self.arc_lengths,
+                        nmpy.hstack((derivatives, last)),
+                    )
+                else:
+                    raise RuntimeError("Should never happen. Please contact developer.")
+            else:
+                arc_lengths = as_curve.x
+                sites, derivatives = (as_curve(arc_lengths, _rdr) for _rdr in (0, 1))
+                norms = nmpy.linalg.norm(derivatives, axis=0, keepdims=True)
+                self._cache[cache_entry] = (sites, arc_lengths, derivatives / norms)
+
+        return self._cache[cache_entry]
+
+    def DirectionChanges(self) -> array_t:
+        """"""
+        cache_entry = self.DirectionChanges.__name__
+
+        if cache_entry not in self._cache:
+            _, arc_lengths, directions = self.Directions()
+            determinants = nmpy.empty((directions.shape[1] - 1), dtype=nmpy.float64)
+            for c_idx in range(directions.shape[1] - 1):
+                determinants[c_idx] = nmpy.linalg.det(
+                    directions[:, c_idx : (c_idx + 2)]
+                )
+            determinants /= arc_lengths[1:]
+
+            self._cache[cache_entry] = determinants
+
+        return self._cache[cache_entry]
+
+    def _InnerSegments(self) -> tuple[tuple[int, ...], tuple[int, ...]]:
+        """"""
+        dim_range = tuple(range(self.dim))
+        first_inner = tuple(
+            self.sites[_idx][1] - self.sites[_idx][0] for _idx in dim_range
+        )
+        last_inner = tuple(
+            self.sites[_idx][-2] - self.sites[_idx][-1] for _idx in dim_range
+        )
+
+        return first_inner, last_inner
+
     def _Direction(self, caller: MethodType, /) -> array_t:
         """"""
         cache_entry = caller.__name__
@@ -152,17 +219,18 @@ class edge_t(raw_edge_t):
 
             as_curve = self.AsCurve()
             if as_curve is None:
-                self._cache[cache_entry_i] = None
-                self._cache[cache_entry_f] = None
+                if self.n_sites > 1:
+                    first_inner, last_inner = self._InnerSegments()
+                    self._cache[cache_entry_i] = nmpy.array(
+                        first_inner
+                    ) / nmpy.linalg.norm(first_inner)
+                    self._cache[cache_entry_f] = nmpy.array(
+                        last_inner
+                    ) / nmpy.linalg.norm(last_inner)
+                else:
+                    raise RuntimeError("Should never happen. Please contact developer.")
             else:
-                dim_range = tuple(range(self.sites.__len__()))
-                first_inner = tuple(
-                    self.sites[_idx][1] - self.sites[_idx][0] for _idx in dim_range
-                )
-                last_inner = tuple(
-                    self.sites[_idx][-2] - self.sites[_idx][-1] for _idx in dim_range
-                )
-
+                first_inner, last_inner = self._InnerSegments()
                 max_arc_length = as_curve.x.item(-1)
                 derivatives = as_curve((0, max_arc_length), 1)
                 initial = derivatives[:, 0]
@@ -182,101 +250,6 @@ class edge_t(raw_edge_t):
 
         return self._cache[cache_entry]
 
-    def Curvatures(self) -> tuple[float, ...]:
-        """"""
-        if self.dim != 2:
-            raise ValueError(
-                f"{self.dim}: Invalid dimension for curvature computation. Expected=2."
-            )
-
-        cache_entry = self.Curvatures.__name__
-
-        if cache_entry not in self._cache:
-            as_curve = self.AsCurve()
-            if as_curve is None:
-                self._cache[cache_entry] = None
-            else:
-                arc_lengths = as_curve.x
-                derivatives_fst, derivatives_scd = tuple(
-                    as_curve(arc_lengths, _rdr) for _rdr in (1, 2)
-                )
-
-                normalization = nmpy.sum(derivatives_fst**2, axis=0) ** 1.5
-                where_safe = normalization != 0.0
-                curvatures = nmpy.empty_like(arc_lengths)
-                curvatures[where_safe] = (
-                    derivatives_fst[0][where_safe] * derivatives_scd[1][where_safe]
-                    - derivatives_fst[1][where_safe] * derivatives_scd[0][where_safe]
-                ) / normalization[where_safe]
-                curvatures[nmpy.logical_not(where_safe)] = 0.0
-
-                self._cache[cache_entry] = tuple(curvatures)
-
-        return self._cache[cache_entry]
-
-    def CurvaturesAndTorsions(self) -> tuple[tuple[float, ...], tuple[float, ...]]:
-        """"""
-        if self.dim != 3:
-            raise ValueError(
-                f"{self.dim}: Invalid dimension for curvature and torsion computation. Expected=3."
-            )
-
-        cache_entry = self.CurvaturesAndTorsions.__name__
-
-        if cache_entry not in self._cache:
-            as_curve = self.AsCurve()
-            if as_curve is None:
-                self._cache[cache_entry] = None
-            else:
-                arc_lengths = as_curve.x
-                derivatives_fst, derivatives_scd, derivatives_trd = tuple(
-                    as_curve(arc_lengths, _rdr) for _rdr in (1, 2, 3)
-                )
-
-                a = (
-                    derivatives_scd[2] * derivatives_fst[1]
-                    - derivatives_scd[1] * derivatives_fst[2]
-                )
-                b = (
-                    derivatives_scd[0] * derivatives_fst[2]
-                    - derivatives_scd[2] * derivatives_fst[0]
-                )
-                c = (
-                    derivatives_scd[1] * derivatives_fst[0]
-                    - derivatives_scd[0] * derivatives_fst[1]
-                )
-                abc_sq_length = a**2 + b**2 + c**2
-
-                normalization = nmpy.sum(derivatives_fst**2, axis=0) ** 1.5
-                where_safe = normalization != 0.0
-                curvatures = nmpy.empty_like(arc_lengths)
-                curvatures[where_safe] = (
-                    nmpy.sqrt(abc_sq_length[where_safe]) / normalization[where_safe]
-                )
-                curvatures[nmpy.logical_not(where_safe)] = 0.0
-
-                where_safe = abc_sq_length != 0.0
-                torsions = nmpy.empty_like(arc_lengths)
-                # Why abs? From: https://mathworld.wolfram.com/Torsion.html
-                #     The torsion is positive for a right-handed curve, [...]
-                # From : https://mathcurve.com/courbes3d.gb/dextre/dextre.shtml
-                #     A right-handed curve is a curve with positive torsion
-                # (!)
-                # Anyway, this explains why abs.
-                torsions[where_safe] = nmpy.abs(
-                    (
-                        derivatives_trd[0][where_safe] * a[where_safe]
-                        + derivatives_trd[1][where_safe] * b[where_safe]
-                        + derivatives_trd[2][where_safe] * c[where_safe]
-                    )
-                    / abc_sq_length[where_safe]
-                )
-                torsions[nmpy.logical_not(where_safe)] = 0.0
-
-                self._cache[cache_entry] = (tuple(curvatures), tuple(torsions))
-
-        return self._cache[cache_entry]
-
     def __str__(self) -> str:
         """
         Does not call raw_edge_t.__str__ since raw edges do not have UID
@@ -291,7 +264,7 @@ class edge_t(raw_edge_t):
 
         main = (
             f"{self.__class__.__name__}.{self.uid}:\n"
-            f"    Sites[{self.dim}-D]: {self.sites[0].size}\n"
+            f"    Sites[{self.dim}-D]: {self.n_sites}\n"
             f"    Origin:              {origin}\n"
             f"    Lengths:             Raw={raw_length}, SB-Area={segment_based_area}"
         )
