@@ -36,11 +36,11 @@ from enum import Enum as enum_t
 from typing import ClassVar
 
 
-node_style_raw_h = tuple[str, int, str]  # type, size, color
+node_style_raw_h = str  # color
 node_styles_raw_h = dict[int | None, node_style_raw_h]
 node_either_raw_h = node_style_raw_h | node_styles_raw_h
 
-edge_style_raw_h = node_style_raw_h
+edge_style_raw_h = tuple[str, int, str]  # type, size, color
 edge_styles_raw_h = tuple[edge_style_raw_h, edge_style_raw_h]
 
 # show, size, color | show (False)
@@ -59,32 +59,22 @@ class plot_mode_e(enum_t):
 
 @dtcl.dataclass(slots=True, repr=False, eq=False)
 class node_style_t:
-    type: str  # E.g., "."
-    size: int  # "markersize"
     color: str  # E.g., "b"
 
-    DEFAULT_TYPE: ClassVar[str] = "."
-    DEFAULT_SIZE: ClassVar[int] = 6
     DEFAULT_COLORS: ClassVar[tuple[tuple[int | None, str]]] = ((None, "g"), (1, "r"))
 
     @classmethod
     def Default(cls) -> node_styles_h:
         """"""
-        return {
-            _key: cls(type=cls.DEFAULT_TYPE, size=cls.DEFAULT_SIZE, color=_vle)
-            for _key, _vle in cls.DEFAULT_COLORS
-        }
+        return {_key: cls(color=_vle) for _key, _vle in cls.DEFAULT_COLORS}
 
     @classmethod
     def NewFromUnstructured(cls, style: node_either_raw_h, /) -> node_styles_h:
         """"""
-        if isinstance(style, tuple):  # node_style_raw_h
-            output = {None: cls(type=style[0], size=style[1], color=style[2])}
+        if isinstance(style, dict):  # node_styles_raw_h
+            output = {_dgr: cls(color=_stl) for _dgr, _stl in style.items()}
         else:
-            output = {
-                _dgr: cls(type=_stl[0], size=_stl[1], color=_stl[2])
-                for _dgr, _stl in style.items()
-            }
+            output = {None: cls(color=style)}
 
         return output
 
